@@ -17,13 +17,12 @@
 package api
 
 import (
-	"io"
-
 	"github.com/docker/cli/cli/streams"
 )
 
+// Streams defines the standard streams (stdin, stdout, stderr) used by the CLI.
 type Streams interface {
 	Out() *streams.Out
-	Err() io.Writer
+	Err() *streams.Out
 	In() *streams.In
 }
